# -*- coding: utf-8 -*-

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('entity_history', '0002_auto_20150406_1605'),
    ]

    operations = []
